import logging
import os

from zygoat.components import SettingsComponent

from .secret_key import secret_key
from .database_config import database_config
from .debug_config import debug_config
from .installed_apps import installed_apps
from .allowed_hosts import allowed_hosts
from .cookies import cookies
from .drf_camelize import drf_camelize
from .reverse_proxy import reverse_proxy
from .env import env
from .security import security
from .settings_file import settings_file

log = logging.getLogger()


class Settings(SettingsComponent):
    def create(self):
        log.info("Making python package for Django settings")
        os.makedirs(self.settings_directory, exist_ok=True)

        log.info("Creating import for zygoat settings in __init__.py")
        with open(os.path.join(self.settings_directory, "__init__.py"), "a") as f:
            f.write(f"from .{SettingsComponent.MODULE_NAME} import *  # noqa\n")

    @property
    def installed(self):
        return os.path.exists(self.settings_directory)

    def update(self):
        red = self.parse()
        key_node = red.find("name", value="SECRET_KEY").parent
        log.info("Retrieving existing secret key")
        default_key = key_node.value
        self.existing_secret_key = default_key


settings_sub_components = [
    settings_file,
    secret_key,
    database_config,
    debug_config,
    installed_apps,
    allowed_hosts,
    cookies,
    security,
    drf_camelize,
    reverse_proxy,
    env,
]

settings = Settings(sub_components=settings_sub_components)



import base64
import logging
import os

from zygoat.components import SettingsComponent

log = logging.getLogger()


class SecretKey(SettingsComponent):
    def create(self):
        red = self.parse()
        key_node = red.find("name", value="SECRET_KEY").parent

        log.info("Generating default secret key")
        default_key = base64.b64encode(os.urandom(32)).decode("utf-8")

        log.info("Relocating default secret key")
        key_node.value = f"prod_required_env('DJANGO_SECRET_KEY', default='{default_key}')"

        log.info("Dumping secret key node")
        self.dump(red)

    def update(self):
        red = self.parse()

        log.info("Copying secret key value from previous settings")
        key_node = red.find("name", value="SECRET_KEY").parent
        key_node.value = self.parent.existing_secret_key

        self.dump(red)

    @property
    def installed(self):
        return bool(getattr(self.parent, "existing_secret_key", None))


secret_key = SecretKey()



import os

from zygoat.constants import Projects
from zygoat.components import Component, FileComponent
from zygoat.components.backend import resources


class ManagementCommand(Component):
    pass


class CommandComponent(FileComponent):
    resource_pkg = resources
    base_path = os.path.join(Projects.BACKEND, "backend", "management", "commands")


class ManagementInit(FileComponent):
    base_path = os.path.join(Projects.BACKEND, "backend", "management")
    filename = "__init__.py"


class CommandInit(FileComponent):
    base_path = os.path.join(Projects.BACKEND, "backend", "management", "commands")
    filename = "__init__.py"


class WaitCommand(CommandComponent):
    filename = "wait_for_db.py"


class StaffCommand(CommandComponent):
    filename = "set_staff.py"


management_command = ManagementCommand(
    sub_components=[ManagementInit(), CommandInit(), WaitCommand(), StaffCommand()]
)



import os

from zygoat.components import FileComponent, SettingsComponent
from zygoat.constants import Projects

from . import resources


class SettingsFile(FileComponent):
    filename = "zygoat_settings.py"
    resource_pkg = resources
    base_path = os.path.join(
        Projects.BACKEND, Projects.BACKEND, SettingsComponent.DIRECTORY_NAME,
    )


settings_file = SettingsFile()



import logging

from zygoat.components import SettingsComponent

log = logging.getLogger()


class Security(SettingsComponent):
    def create(self):
        red = self.parse()

        log.info("Adding security headers")

        red.extend(
            [
                "\n",
                "# Set security headers",
                "X_FRAME_OPTIONS = 'DENY'",
                "SECURE_BROWSER_XSS_FILTER = True",
                "SECURE_CONTENT_TYPE_NOSNIFF = True",
                "SECURE_HSTS_SECONDS = 31536000",
                "SECURE_HSTS_INCLUDE_SUBDOMAINS = True",
                "\n",
            ]
        )
        self.dump(red)

    @property
    def installed(self):
        red = self.parse()
        return red.find("name", value="SECURE_HSTS_SECONDS") is not None


security = Security()
